/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_LinAlg.cpp 
Usage: Contains various usefull function implementations
 Last Modified: 1/12/2010
=========================================================================================================
*/
#include <math.h>
#include <stdlib.h>
#include "SOURCE_params.h"


/*
 * =======================================================
    N O R M A L L Y  D I S T R I B U T E D   N U M B E R S
 * =======================================================
 */
double normal_distrib()
{
	double 	u1, u2, n1, n2, pi=3.14159265358979;

	u1 = drand48();
	u2 = drand48();
	n1 = sqrt(-2.0*log(u1))*cos(2.0*pi*u2);
	n2 = sqrt(-2.0*log(u1))*sin(2.0*pi*u2);
	u1 = drand48();
	if (u1 > 0.5)
		return(n1);
	else
		return(n2);
}

/*
 * =========================
    V E C T O R   N O R M S
 * =========================
 */

double norms (int dim, double X[], int WhichNorm)
{
	int i;
	double	sum;
	
	
	if (WhichNorm == 1)				// L_1 NORM => SUM(ABS(X))
	{
		sum = 0.0;
		for (i=0; i<dim; i++)
			sum = sum + fabs(X[i]);
	}
	else if (WhichNorm == 2)			// L_2 NORM => SQRT(SUM(X^2))
	{
		sum = 0.0;
		for (i=0; i<dim; i++)
		        sum = sum + X[i]*X[i];
		sum = sqrt(sum);
	}
	else if (WhichNorm == 3)                        // L_inf NORM => MAX(ABS(X))
	{
		sum = fabs(X[0]);
		for (i=1; i<dim; i++)
		{
			if (fabs(X[i]) > sum)
				sum = fabs(X[i]);
		}
	}
	else
	{
		exit(printf("ERROR in norms: Unknown norm identifier %i \n", WhichNorm));	
	}
	return(sum);
}

/*
 * ========================================================
   N O R M  O F  A  D I F F E R E N C E  O F  V E C T O R S   
 * ========================================================
 */
double norm_diff(int dim, double X[], double Y[], int WhichNorm)
{
	double *temp, res;
	int i;
	
	temp = (double *) malloc(dim*sizeof(double));
	for (i=0; i<dim; i++)
		temp[i] = X[i] - Y[i];

	res  = 	norms(dim, temp, WhichNorm);	
	free(temp);
	return(res);	
	
	
}

/*
 * ========================
   C O P Y   V E C T O R S
 * ========================
 */

// DOUBLE VERSION
void copy_vector (int size, double X[], double Y[])
{
	int i;
	
	for (i=0; i<size; i++)
		Y[i] = X[i];
}

// INTEGER VERSION
void copy_vector_i (int size, int X[], int Y[])
{
	int i;
	
	for (i=0; i<size; i++)
		Y[i] = X[i];
}

/*
 * ===========================================================
   R A N D O M   P E R M U T A T I O N   O F   I N T E G E R S
 * ===========================================================
 */

void randperm ( int dim, int V[] )
{
       int i, j;

       for ( i=0; i<dim; ++i ) 
       {
               j = rand() % (i + 1);
               V[i] = V[j];
               V[j] = i;

       }
}

// ********************************
// M E A N   V A L U E
// ********************************

double mean (int N, double Y[])
{
// INPUT ARGUMENTS
// ---------------
// N : dimension of input vector
// Y : input vector

        int i;
        double yy;

        yy = 0.0;
        for (i=0; i<N; i++)
                yy += Y[i];
        yy = yy/(double) N;
        return(yy);
}

double stdev (int N, double Y[])
{
// INPUT ARGUMENTS
// ---------------
// N : dimension of input vector
// Y : input vector

        int i;
        double aux, yy;

        aux = mean(N,Y);
        yy = 0.0;
        for (i=0; i<N; i++)
                yy += (Y[i]-aux)*(Y[i]-aux);
        yy = sqrt(yy/(N-1));
        return(yy);
}


double mat_max (int N, int M, double Y[])
{
/*
 Returns the maximum absolute value of matrix Y

 INPUT ARGUMENTS
 ---------------
 N : number of rows
 M : number of columns
 Y : input matrix (NxM)
*/
        int i,j;
        double mmax = fabs(Y[0]);
	

        for (i=0;i<M;i++) {
                for (j=0;j<N;j++) 
                {
			if (mmax < fabs(Y[i*N+j]))
                		mmax = fabs(Y[i*N+j]);   
                }
        }
        return mmax;
}

double euclidean_std (int N, int M, double Y[])
{
/*
 Returns a vector with the standard deviations of Y[] in columns

 INPUT ARGUMENTS
 ---------------
 N : number of rows dim
 M : number of columns SS 
 Y : input matrix (NxM)
*/
        int i,j;
        double aux[M], res = 0.0;
	double mean[N];


	for (j=0; j<N; j++)
		mean[j] = 0.0;

        for (i=0;i<M;i++) {
                for (j=0;j<N;j++) 
                {
                	mean[j] = mean[j] + Y[i*N+j];   
                }
        }

	for (j=0; j<N; j++)
		mean[j] = mean[j] / (double)M; 

	res = 0.0;
	for (i=0; i<M; i++)
	{
		aux[i] = 0.0;
		for (j=0; j<N; j++)
		{
			aux[i] = aux[i] + (mean[j]-Y[i*N+j])*(mean[j]-Y[i*N+j]);
		}
		res = res + aux[i];
	}

	res = sqrt(res / (double)M);
	        
        return res;
}

double mat_stdev (int N, int M, double Y[])
{
/*
 Returns a vector with the standard deviations of Y[] in columns

 INPUT ARGUMENTS
 ---------------
 N : number of rows
 M : number of columns
 Y : input matrix (NxM)
*/
        int i,j;
        double aux[N], STDs = 0.0;

        for (i=0;i<M;i++) {
                for (j=0;j<N;j++) 
                {
                	aux[j] = Y[i*N+j];    // i-th column of Y[]
                }
                STDs += stdev (N, aux);               // Standard deviation of i-th column
        }
	        
        return STDs/(double)N;
}

double mat_mean(int N, int M, double Y[])
{
       int i,j;
        double aux[N], MEANs = 0.0;

        for (i=0;i<M;i++) {
                for (j=0;j<N;j++) 
                {
                	aux[j] = Y[i*N+j];    // i-th column of Y[]
                }
                MEANs += mean (N, aux);      
        }
        return (MEANs/(double)M);
}

