/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_StopCriterion.cpp 
Usage: Implements stopping criterion for PSO and DE algorithms
 Last Modified: 1/12/2010
=========================================================================================================
*/
#include "SOURCE_params.h"

/*
 --------------------------------------
  STOPPING CRITERION 
  Algorithm stops when reaching a 
  predefined number of function
  evaluations or iterations
 -------------------------------------
 
 Input: 
	PROBLEM_par prob - A structure that contains problem depended parameters
	RUN_STATS *stats - A structure that contains specific running parameters
 Output:
	1: Stop the algorithm, the criterion is satisfied
	0: Continue the algorithm, the criterion is not met
*/

int StopCriterion(PROBLEM_par prob, RUN_STATS *stats)
{

	if (prob.maxfev != -1 && stats->fev>=prob.maxfev)
		return(1);

	if ( prob.iVTR != -1 &&  fabs(stats->bestval - prob.VTR) / max(1.0, fabs(stats->bestval)) < FEPS)
		return 1;		

	if (prob.maxgev != -1 &&  stats->gev >= prob.maxgev)
		return 1;

	if (prob.maxit != -1 &&  stats->iter >= prob.maxit)
		return 1;

	return 0;

}

