/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_UPSO_DE.c 
 Usage: Main UPSO and DE subroutines
 Last Modified: 1/12/2010
=========================================================================================================
*/

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "SOURCE_params.h"
#include "SOURCE_UPSO_DE.h"

int override_bounds = 0;

//-------------------


void init_mempsode()
{
        stats.bestind  = 0;
        stats.bestval  = 0;
        stats.success  = 0;
        stats.iter     = 1;
        stats.fev      = 0;
        stats.lasthit  = 0;
        stats.BPup     = 0;
        stats.localsearch      = 0;
        stats.gev      = 0;

        algo.algi      = 1;
        prob.aa        = 0;
        prob.dim       = 2;
        algo.SS        = 10 * prob.dim;
        prob.maxfev    = -1;
        prob.maxgev    = -1;
        prob.maxit     = -1;
        prob.VTR       = -INF;
        prob.iVTR      = -1;
        prob.nexp      = 1;
        algo.nradius   = 1;
        algo.UF        = 1.0;
        algo.chi = 0.729;
        algo.c1  = 2.05;
        algo.c2  = 2.05;
        algo.seed      = 1;
        algo.R3use     = 0;
        algo.R3mean    = 0.0;
        algo.R3std     = 1.0;
        algo.F         = 0.5;
        algo.CR        = 0.7;
        algo.oper      = 1;
        algo.memetic   = 0;
        algo.stopping  = 0;
        algo.Vscale    = 0.1;
        algo.restart   = 0;
        algo.dump_swarm = 0;
        algo.new_vel    = 0;
        algo.prob      = 0.01;
        algo.dump      = 50;
        EPS     = 1.0e-8;
        XEPS    = 1.0e-8;
        FEPS    = 1.0e-6;
        GEPS    = 1.0e-5;
    	strcpy(TMPIN, "in.dat");
        strcpy(TMPOUT, "out.dat");
        strcpy(TMPINFILE, ".psode");
        strcpy(TMPOUFILE, ".psode");
        algo.seed = (int)time(NULL);
        srand48((time_t)algo.seed);
        noprint = 0;

}


void set_mempsode_iparam(char *name, int *val)
{
	if (strcmp(name, "algorithm") == 0)
	{
		if (*val == 1 || *val == 2) 
			algo.algi = *val;
		else
			printf("Warning: invalid 'algorithm' value \n");
	}
	else if (strcmp(name, "objective") == 0)
	{
		if (*val >= 0) 
			prob.aa = *val;
		else 
			printf("Warning: invalid 'objective' value \n");
	}
	else if (strcmp(name, "dimension") == 0)
	{
		if (*val >= 0) 
			prob.dim = *val;
		else 
			printf("Warning: invalid 'dimension' value \n");
	}
	else if (strcmp(name, "swarm-size") == 0)
        {
                if (*val >= 0) 
                        algo.SS = *val;
                else
                        printf("Warning: invalid 'swarm-size' value \n");
        }
	else if (strcmp(name, "max-fun-evals") == 0)
        {
                if (*val >= 0) 
                        prob.maxfev = *val;
                else
                        printf("Warning: invalid 'max-fun-evals' value \n");
        }
	else if (strcmp(name, "max-grad-evals") == 0)
        {
                if (*val >= 0) 
                        prob.maxgev = *val;
                else
                        printf("Warning: invalid 'max-grad-evals' value \n");
        }
	else if (strcmp(name, "max-iter") == 0)
        {
                if (*val >= 0) 
                        prob.maxit = *val;
                else
                        printf("Warning: invalid 'max-iter' value \n");
        }
	else if (strcmp(name, "radius") == 0)
        {
                if (*val >= 1) 
                        algo.nradius = *val;
                else
                        printf("Warning: invalid 'radius' value \n");
        }
	else if (strcmp(name, "seed") == 0)
        {
                if (*val >= 0) 
		{
			algo.seed = *val;
		        srand48((time_t)algo.seed);
		}
                else
                        printf("Warning: invalid 'radius' value \n");
        }
	else if (strcmp(name, "memetic") == 0)
        {
                if (*val == 0 || *val == 1 || *val == 2 || *val == 3) 
		{
			algo.memetic = *val;
		}
                else
                        printf("Warning: invalid 'memetic' value \n");
	}
	else if (strcmp(name, "dump-iterations") == 0)
        {
                if (*val > 0 )
			algo.dump = *val;
                else
                        printf("Warning: invalid 'dump-iterations' value \n");
	}
	else if (strcmp(name, "pso-params") == 0)
        {
                if (*val == 1 )
		{
			algo.chi = 0.729;
        		algo.c1  = 2.05;
        		algo.c2  = 2.05;
		}
		else if (*val == 2)
		{
			algo.chi = 0.729;
        		algo.c1  = 2.05;
        		algo.c2  = 2.05;
		}
                else
                        printf("Warning: invalid 'pso-params' value \n");
	}
	else if (strcmp(name, "use-mutation") == 0)
        {
                if (*val == 1 || *val == 0 )
			algo.R3use = *val;
                else
                        printf("Warning: invalid 'use-mutation' value \n");
	}
	else if (strcmp(name, "de-strategy") == 0)
        {
                if (*val == 1 || *val == 2 || *val == 3 || *val == 4 || *val == 5 )
			algo.oper = *val;
                else
                        printf("Warning: invalid 'de-strategy' value \n");
	}
	else if (strcmp(name, "file-output") == 0)
        {
                if (*val == 0 || *val == 1)
			noprint = *val;
                else
                        printf("Warning: invalid 'file-output' value \n");
	}

}

void set_mempsode_dparam(char *name, double *val)
{
	int i;
        if (strcmp(name, "target-value") == 0)
        {
        	prob.VTR = *val;
        	prob.iVTR = 1;
        }
        else if (strcmp(name, "unification-factor") == 0)
        {
		if (*val >= 0.0 && *val <= 1.0)
        		algo.UF = *val;
		else
			printf("Warning: invalid 'unification-factor' value \n");
        }
        else if (strcmp(name, "mutation-mean") == 0)
        {
		if (*val >= 0.0 && *val <= 1.0)
        		algo.R3mean = *val;
		else
			printf("Warning: invalid 'mutation-mean' value \n");
        }
        else if (strcmp(name, "mutation-std") == 0)
        {
		if (*val >= 0.0)
        		algo.R3std = *val;
		else
			printf("Warning: invalid 'mutation-std' value \n");
        }
        else if (strcmp(name, "F-param") == 0)
        {
		if (*val >= 0.0)
        		algo.F = *val;
		else
			printf("Warning: invalid 'F-param' value \n");
        }
        else if (strcmp(name, "CR-param") == 0)
        {
		if (*val >= 0.0)
        		algo.CR = *val;
		else
			printf("Warning: invalid 'CR-param' value \n");
        }
        else if (strcmp(name, "velocity-scaling") == 0)
        {
		if (*val > 0.0)
        		algo.Vscale = *val;
		else
			printf("Warning: invalid 'velocity-scaling' value \n");
        }
        else if (strcmp(name, "prob-local-search") == 0)
        {
		if (*val >= 0.0 && *val <= 1.0)
        		algo.prob = *val;
		else
			printf("Warning: invalid 'prob-local-search' value \n");
        }
        else if (strcmp(name, "EPS") == 0)
        {
		if (*val > 0.0 )
        		EPS = *val;
		else
			printf("Warning: invalid 'EPS' value \n");
        }
        else if (strcmp(name, "XEPS") == 0)
        {
		if (*val > 0.0 )
        		XEPS = *val;
		else
			printf("Warning: invalid 'XEPS' value \n");
        }
        else if (strcmp(name, "FEPS") == 0)
        {
		if (*val > 0.0 )
        		FEPS = *val;
		else
			printf("Warning: invalid 'FEPS' value \n");
        }
        else if (strcmp(name, "GEPS") == 0)
        {
		if (*val > 0.0 )
        		GEPS = *val;
		else
			printf("Warning: invalid 'GEPS' value \n");
        }
  	else if (strcmp(name, "XMIN") == 0)
        {
		override_bounds++;
		for (i=0; i<prob.dim; i++)
			prob.Xmin[i] = val[i];
        }
  	else if (strcmp(name, "XMAX") == 0)
        {
		override_bounds++;
		for (i=0; i<prob.dim; i++)
			prob.Xmax[i] = val[i];
        }




}

void set_mempsode_cparam(char *name, char *val)
{
	if (strcmp(name, "algorithm") == 0)
        {
                if (strcmp(val, "pso") == 0)
                        algo.algi = 1;
                else if (strcmp(val, "de") == 0)
                        algo.algi = 2;
                else
                        printf("Warning: invalid 'algorithm' value \n");
        }
	else if (strcmp(name, "infile") == 0)
        {
                       strcpy(TMPIN, val); 
        }
	else if (strcmp(name, "outfile") == 0)
        {
                       strcpy(TMPOUT, val); 
        }
}


void get_mempsode_dparam(char *name, double *val)
{
	int i;

        if (strcmp(name, "minimum") == 0)
        {
        	*val = stats.bestval;
        }
        else if (strcmp(name, "minimizer") == 0)
        {
		for (i=0; i<prob.dim; i++)
        		val[i] = stats.best_x[i];
        }
}

void mempsode()
{
	int i;
	vel             = (double *) malloc(algo.SS*prob.dim*sizeof(double));
        swarm           = (double *) malloc(algo.SS*prob.dim*sizeof(double));
        fswarm          = (double *) malloc(algo.SS*sizeof(double));
        bestpos         = (double *) malloc(algo.SS*prob.dim*sizeof(double));
        fbestpos        = (double *) malloc(algo.SS*sizeof(double));
        stats.best_x 	= (double *) malloc(prob.dim*sizeof(double));
        algo.Vmax       = (double *) malloc(prob.dim*sizeof(double));
	prob.Xmin       = (double *) malloc(prob.dim*sizeof(double));
        prob.Xmax       = (double *) malloc(prob.dim*sizeof(double));


	stats.bestind  = 0;
        stats.bestval  = 0;
        stats.success  = 0;
        stats.iter     = 1;
        stats.fev      = 0;
        stats.lasthit  = 0;
        stats.BPup     = 0;
        stats.localsearch      = 0;
        stats.gev          = 0;

	if (override_bounds < 2)	
		Bounds_F(prob.Xmin, prob.Xmax, prob.dim);

  	for (i=0; i<prob.dim; i++)
                algo.Vmax[i] = (prob.Xmax[i]-prob.Xmin[i])*algo.Vscale;


	UPSO_DE(swarm, fswarm, vel, bestpos, fbestpos, algo, prob, &stats, algo.algi);
}

