/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_local_opt.cpp 
Usage: Wrapper routine that calls Merlin's local optimizers through OPTIMA interface
 Last Modified: 1/12/2010
=========================================================================================================
*/
#include <stdlib.h>
#include "SOURCE_params.h"
#include "SOURCE_local_opt.h"

/*
 ------------------------------------------------------
 PERFORM LOCAL OPTIMIZATION BY CALLING OPTIMA INTERFACE  
 ------------------------------------------------------

 Input: 
	double	 	*in_x 	- Starting point  
	int 		n	- Problem's dimension
	double		*xll	- Problem's lower bounds  	
	double		*xrl	- Problem's upper bounds
	
	
 Output:
	double 		*out_x	- Ending  point of the local minimization 
	double 		*out_val- Objective function value at "out_x"
	double 		*grms	- Euclidean norm of the gradient vector at "out_x"
	int 		*fc	- Number of function evaluations performed
	int 		*gc	- Number of gradient evaluations performed
*/
#ifdef NOMERLIN
void local_optimization(double *in_x, int n, double *out_x, double *out_val, double *grms, 
			double *xll, double *xrl, int *fc, int *gc)
{
		/* void wrapper */
}
void  chemin_ ( double *GRMS, double *EPS, double *TP, int *NOD, double *VAL, double *xll, double *xrl, double *GRAD, int *NOC, int *NGR )
{
		/* void wrapper */
}
#else
void local_optimization(double *in_x, int n, double *out_x, double *out_val, double *grms, 
			double *xll, double *xrl, int *fc, int *gc)
{
	int i, NOD,NOF, *IXAT, ICODE[4];
	int NF, NG, NH, NJ, TMP1, TMP2, NOC, NGR;
	double VAL, *TP, *GRAD, step;	
	double GRMS;
	FILE *tmpf;

	
	IXAT = (int *)malloc(n*sizeof(int));
	TP = (double *)malloc(n*sizeof(double));
	GRAD = (double *)malloc(n*sizeof(double));

	if ((tmpf = fopen(TMPIN, "r")) == NULL)
		exit(printf("File %s containing Merlin commands does not exist\n", TMPIN));
	fclose(tmpf);

	
	NF = 0;
	NG = 0;
	NOD = n; 
	NOF = 0; 
	for (i=0; i<n; i++)
	{
		TP[i] = in_x[i];
		IXAT[i] = 1;
	}
	Objective_F (TP, n, &VAL);
	GRMS = 1.0;
	ICODE[0] = 1;
	ICODE[1] = 1;
	ICODE[2] = 1;
	ICODE[3] = 0;
	TMP1 = strlen(TMPIN);
	TMP2 = strlen(TMPOUT);
#ifdef TINKER
	ICODE[1] = 0; 
	ICODE[2] = 0; 
#endif	

	/*
	 *
	  Call Merlin's OPTIMA interface to access local optimization subroutines
	 *
	 */
	wunform_(TP,&NOD);


	optima_(&NOD,&NOF,TP,&VAL,xll,xrl,IXAT,ICODE,TMPIN, TMPOUT, &GRMS,&NF,&NG,&NH,&NJ, TMP1, TMP2);

	NOC = 0;
	chemin_ ( &GRMS, &EPS, TP, &NOD, &VAL, xll, xrl, GRAD, &NOC, &NGR );

	for (i=0; i<n; i++)
	{
		out_x[i] = TP[i];
	}
#ifdef TINKER
	for (i=0; i<NOD; i++)
	{
		if (out_x[i] > 10.e6)
		{
			step = floor(out_x[i] / 360.0);
			out_x[i] = out_x[i] - (step-1)*360.0;
		}
		while (out_x[i] > 180.0)
			out_x[i] = out_x[i] - 360.0;
	}
	for (i=0; i<NOD; i++)
	{
		if (out_x[i] < -10.e6)
		{
			step = floor(fabs(out_x[i]) / 360.0);
			out_x[i] = out_x[i] + (step-1)*360.0;
		}
		while (out_x[i] < -180.0)
			out_x[i] = out_x[i] + 360.0;
	}
#endif
	*out_val = VAL;
	*grms = GRMS;
	*fc = NF + NOC ;
	*gc = NG;

	free(IXAT);
	free(TP);
	free(GRAD);
}
#endif
