/*
   =========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_main.c 
 Usage: Main function, parse arguments, open files for printouts
 Last Modified: 1/12/2010
=========================================================================================================
*/
/*
=========================================

   M A I N   P R O G R A M   F O R 
 
   U N I F I E D   P S O   A N D   D E

=========================================
*/
 
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <string.h>


/*
=========================================

 PARAMETER STRUCTURES
 
========================================= 
*/
#include "SOURCE_params.h"
#include "SOURCE_main.h"

extern setfla_(int *t_flag, double *t_value);

#ifndef TINKER
void granalize_(int *N, double *X, double *G)
{
        //
}
#endif
#ifndef GRAD
void Objective_G(double X[], int N, double *G)
{
	//
}
#endif 

#ifdef NOMERLIN
 struct{
     int ipro;
}funcid_;
#else
  extern struct{
     int ipro;
}funcid_;
#endif

/*
=========================================

 MAIN PROGRAM
 
========================================= 
*/
int main(int argc, char **argv) 
{
	//--------------------------------------
	// DEFINE AND READ STRUCTURE PARAMETERS
	//--------------------------------------
/*	ALGO_par	algo;
	PROBLEM_par	prob;
	RUN_STATS	stats;*/
	RUN_STATS	*experiments;
	int		PSOparset;
	int 		t_flag;
	int		i_min_f;
	double		min_f;
	double 		t_value;
	clock_t		start,end;
	char 		foutpre[50];
	char 		foutint[50], foutname1[130], foutname2[130], foutname3[130], foutname4[130];
	FILE 		*fout1, *fout2, *fout3;


	//-----------------------------------
	// DEFINE NUMBER OF INPUT PARAMETERS
	//-----------------------------------
	int	algi, i, j;

	//-----------------------------------
	// DEFAULT VALUES FOR PARAMETERS
	//-----------------------------------	
   	parse_arguments(argc, argv, &algi, &prob, &algo, &PSOparset, foutpre, &noprint, &printsolflag);


	//-----------------------------------------------
	// Define various parameters
	//-----------------------------------------------
   	funcid_.ipro  = prob.aa;
 	pflag_.iflag = 0;
	


	// RANDOM NUMBER GENERATOR
	if (algo.seed == -1)
	{	
		algo.seed = (int)time(NULL);	
		srand48((time_t)algo.seed);
	}
	else
	{
		srand48((time_t)algo.seed);
	}
	// PSO chi, c1, c2
	if (algi==1) 
	{
	
		if (PSOparset==1) 
		{
			algo.chi = 0.729;
			algo.c1  = 2.05;
			algo.c2  = 2.05;
		} 
		else 
		{
			algo.chi = 0.6;
			algo.c1  = 2.83;
			algo.c2  = 2.83;
		}

	} 

	//----------------------------
	// DEFINE ALL DYNAMIC VECTORS
	//----------------------------
        vel     	= (double *) malloc(algo.SS*prob.dim*sizeof(double));
        swarm   	= (double *) malloc(algo.SS*prob.dim*sizeof(double));
	fswarm      	= (double *) malloc(algo.SS*sizeof(double));
	bestpos      	= (double *) malloc(algo.SS*prob.dim*sizeof(double));
	fbestpos      	= (double *) malloc(algo.SS*sizeof(double));
	experiments	= (RUN_STATS *) malloc(prob.nexp * sizeof(RUN_STATS));
	for (i=0; i<prob.nexp; i++)
		experiments[i].best_x = (double *) malloc(prob.dim*sizeof(double));
	stats.best_x = (double *) malloc(prob.dim*sizeof(double));
	
		
	prob.Xmin 	= (double *) malloc(prob.dim*sizeof(double));
	prob.Xmax 	= (double *) malloc(prob.dim*sizeof(double));
	
	algo.Vmax 	= (double *) malloc(prob.dim*sizeof(double));

	//---------------------------
	// DEFINE BOUNDS AND TARGET VALUE
	//---------------------------
	Bounds_F(prob.Xmin, prob.Xmax, prob.dim);

	//---------------------------
	// RESTART THE SWARM USING 
	// PREVIOUSLY DUMPED DATA 
	//---------------------------
	if (algo.restart == 1)
	{
		read_swarm(swarm, fswarm, bestpos, fbestpos, vel, algo, prob, &stats);
	}

	//---------------------------
	// DEFINE INPUT/OUTPUT FILES
	//---------------------------
	if (noprint == 1 )
	{
	
		strcpy(foutname1,foutpre);						
		strcpy(foutname2,foutpre);						
		sprintf(foutint,"_Alg%d_Fun%d_Dim%d_Swarm%d_Mem%d_Fmax%li_", algo.algi, prob.aa, prob.dim, algo.SS, algo.memetic, prob.maxfev);	
		strcat(foutname1,foutint);						
		strcat(foutname2,foutint);
	
		if (algi==1) {
			sprintf(foutint,"UF%3.1f_R3use%d_", algo.UF, algo.R3use);
		} else {
			sprintf(foutint,"OP%d_", algo.oper);
		}
		strcat(foutname1,foutint);						
		strcat(foutname2,foutint);
		strcpy(foutname3,foutname1);
		strcpy(foutname4,foutname1);
		strcat(foutname1,"ver");						
		strcat(foutname2,"rep");
		strcat(foutname3,"sol");
		strcat(foutname4,"tmp");

	
	//---------------------------------
	// INITIALIZE PARAMETERS AND FILES
	//---------------------------------
	
		// OUTPUT FILES
		fout1 = fopen(foutname1, "w");					
		if (fout1 == NULL)
			exit(printf("Cannot open file for writing \n"));
		fout2 = fopen(foutname2, "w");
		if (fout2 == NULL)
			exit(printf("Cannot open file for writing \n"));
		fout3 = fopen(foutname3, "w");
		if (fout3 == NULL)
			exit(printf("Cannot open file for writing \n"));
		algo.tmpfile = fopen(foutname4, "w");
		if (algo.tmpfile == NULL)
			exit(printf("Cannot open file for writing \n"));
	}	
	
	// MAXIMUM VELOCITY
	for (i=0; i<prob.dim; i++)
		algo.Vmax[i] = (prob.Xmax[i]-prob.Xmin[i])*algo.Vscale;
	
	//-------------------------------------
	// WRITE PREAMBLES ON SCREEN & IN FILE
	//-------------------------------------
	if (noprint == 1)
		FPreamble (fout1, algo,  prob, algi);
	

	
	//=========================================
	//=========================================
	// LOOP ON THE TOTAL NUMBER OF EXPERIMENTS
	//=========================================
	//=========================================
	min_f 	= 1.e+100;
	i_min_f = -1;	
	for (i=0; i<prob.nexp; i++) {
		
		
		//------------------------------
		// INITIALIZE CURRENT RUN STATS
		//------------------------------
		if (algo.restart == 0)
		{
			stats.bestind 	= 0;
			stats.bestval 	= 0;
		}
		stats.success 		= 0;
		stats.iter    		= 1;
		stats.fev     		= 0;
		stats.lasthit 		= 0;
		stats.BPup    		= 0;
		stats.localsearch    	= 0;
		stats.gev	    	= 0;
		
		
		if (algo.dump > 0)
		{
			printf("----------------------------------\n");
			printf("Experiment %i \n", i);
			printf("----------------------------------\n");
		}
		
		// Wall clock time
		start = clock();

		UPSO_DE(swarm, fswarm, vel, bestpos, fbestpos, algo, prob, &stats, algi);

		end   = clock();
		if (algo.dump > 0)
			printf("----------------------------------\n");			
		
		// SIMPLE ERROR CHECK
		//--------------------
		if (fabs(stats.bestval - fbestpos[stats.bestind]) > FEPS)
			printf("ERROR IN MAIN.CPP: RESULTS DO NOT MATCH! %.12le - %.12le \n\n", stats.bestval, fbestpos[stats.bestind]);
			
		stats.cpu     = ((double) (end-start))/CLOCKS_PER_SEC;

		experiments[i].success = stats.success;
		experiments[i].iter = stats.iter;
		experiments[i].fev = stats.fev;
		experiments[i].BPup = stats.BPup;
		experiments[i].lasthit = stats.lasthit;
		experiments[i].cpu = stats.cpu;
		experiments[i].localsearch = stats.localsearch;
		experiments[i].gev = stats.gev;
		experiments[i].lastf = stats.lastf;
		experiments[i].lastg = stats.lastg;
		experiments[i].lastl = stats.lastl;
					
		experiments[i].best_f = fbestpos[stats.bestind];
		if  (experiments[i].best_f < min_f) 
		{
		       i_min_f = i;	
		       min_f = experiments[i].best_f;
		}
		for (j=0; j<prob.dim; j++)
				experiments[i].best_x[j] = bestpos[stats.bestind*prob.dim+j];
			
	} // END LOOP ON EXPERIMENTS
	SPreamble (algo,  prob, algi);
	for (i=0; i<prob.nexp; i++) 
	{
			write_results(i, prob, experiments[i], fout1, fout2, fout3, noprint);
	}
#ifdef TINKER				
	// IF TINKER ENABLED OUTPUT .INT FILE CONTAINING THE SOLUTION
	printf("Output tinker file \n");
	pflag_.iflag = 1;
	t_flag = 1;
	t_value = 1.0;
   	//setfla_(&t_flag, &t_value);		
	Objective_F (experiments[i_min_f].best_x, prob.dim, &t_value);
#endif
	
	//--------------------
	// CLOSE OUTPUT FILES
	//--------------------
	
	printf("====================================================================================================================\n");

	if (noprint == 1 )
	{	
		fprintf(fout1,"====================================================================================================================\n");
		fclose(fout1);
		fclose(fout2);
		fclose(fout3);
	}

	//----------------------------
	// WRITE SWARM POSISTIONS AND 
	// VELOCITIES TO TEMPORARY FILE
	// ---------------------------
	if (algo.dump_swarm == 1)
		dump_swarm(swarm, fswarm, bestpos, fbestpos, vel, algo, prob, stats);

	printf("Execution Complete \n");	

	free (vel);
	free(swarm);
	free(fswarm);
	free(bestpos);
	free(fbestpos);
	free(prob.Xmin);
	free(prob.Xmax);
	free(algo.Vmax);

	return 0;	
} // END PROGRAM


/*
 ---------------------------------------------
  Parse Arguments
  
  Performs command line parsing and sets the 
  default parameters of the algorithms.
 ---------------------------------------------
 
 Input: 
	int argc 		- The number of arguments
	char **argv, 		- An array holding the arguments
 Output:
	int *algi		- Switch between UPSO (1) and DE (2)
	PROBLEM_par *prob, 	- Problem depended parameters
	ALGO_par *algo, 	- Algorithm depended parameters
	int *PSOparset, 	- Which set of UPSO updating parameters the user 
				  has requested
	char *foutpre,
	int *noprint, 
	int *printsolflag

*/

int parse_arguments(int argc, char **argv, int *algi, PROBLEM_par *prob, ALGO_par *algo, int *PSOparset, char *foutpre, int *noprint, int *printsolflag)
{
	char c;
	int option_index;
	FILE	*fp;
	*algi 		= 1;
	algo->algi 	= 1;
	prob->aa 	= 0;
	prob->dim 	= 2;
	algo->SS 	= 10 * prob->dim;
	prob->maxfev 	= -1;
	prob->maxgev 	= -1;
	prob->maxit 	= -1;
	prob->VTR	= -INF;
	prob->iVTR	= -1;
	prob->nexp 	= 1;
	algo->nradius 	= 1;
	algo->UF 	= 1.0;
	*PSOparset 	= 1;
	algo->seed 	= 1;
	algo->R3use 	= 0;
	algo->R3mean 	= 0.0;
	algo->R3std 	= 1.0;
	algo->F 	= 0.5;
	algo->CR 	= 0.7;
	algo->oper 	= 1;
	algo->memetic 	= 0;
	algo->stopping  = 0;
	algo->Vscale 	= 0.1;
	algo->restart   = 0;
	algo->dump_swarm = 0;
	algo->new_vel    = 0;	
	strcpy(foutpre, "Default");
	*noprint = 1;	
	*printsolflag = 0;
    	opterr = 0;
    	algo->prob 	= 0.01;
    	algo->dump	= 50;
	EPS	= 1.0e-8;
	XEPS	= 1.0e-8;
	FEPS	= 1.0e-6;
	GEPS 	= 1.0e-5;
    	strcpy(TMPIN, "in.dat");
  	strcpy(TMPOUT, "out.dat");
	strcpy(TMPINFILE, ".psode");	
	strcpy(TMPOUFILE, ".psode");	
       

       while ((c = getopt_long (argc, argv, "a:o:d:s:f:e:r:u:t:n:U:M:S:F:C:v:O:l:i:p:c:P:D:E:Y:X:y:z:R:W:g:b:G:xhV", 
		          long_options, &option_index)) != -1)
       	switch (c)
           {
	   case -1:	    
		break;
           case 'a':
           {	
	      	if (strcmp(optarg, "pso") == 0)
	 	{
			*algi = 1; algo->algi = 1;
		}
		else if (strcmp(optarg, "de") == 0)
		{
			*algi = 2; algo->algi = 2;
		}
		else
		{
			printf("usage: %s -a (pso/de) \n", argv[0]);
			exit(1);
		}
        break;
        }
        case 'o':
	   {
		prob->aa = atoi(optarg); 
        break;
	   }
	  case 'd':
	   {
		prob->dim = atoi(optarg);
		if (prob->dim == 0)
		{
			printf("-d: dimension must be an integer > 0 \n");
			exit(1);
		}
		break;
	   }
	  case 's':
	   {
		algo->SS = atoi(optarg);
		if (algo->SS == 0)
		{
			printf("-s: swarm size must be an integer > 0 \n");
			exit(1);		
		}
		break;
	   }
	  case 'f':
	   {
		prob->maxfev = atoi(optarg);
		if (prob->maxfev <= 0)
		{
			printf("-f: maximum function evaluations must be an integer > 0 \n");
			exit(1);
		}
		break;
	   }
	  case 'g':
	   {
		prob->maxgev = atoi(optarg);
		if (prob->maxgev <= 0)
		{
			printf("-g: maximum gradient evaluations must be an integer > 0 \n");
			exit(1);
		}
		break;
	   }
	   case 'i':
	   {
	  	prob->maxit = atoi(optarg);
	  	if (prob->maxit <= 0 )
	   	{
			printf("-i: maximum number of iterations must be an integer > 0 \n");
			exit(1);			
		}
	        break;
	   }	
	   case 't':
	   {
	  	prob->VTR = atof(optarg);
	  	prob->iVTR = 1;
		break;
	   }	
	  case 'e':
	   {
		prob->nexp = atoi(optarg);
		if (prob->nexp == 0)
		{
			printf("-e: number of experiments must be an integer > 0 \n");			
			exit(1);
		}
		break;
	   }
	  case 'r':
	   {
		algo->nradius = atoi(optarg);
		if (algo->nradius == 0)
		{
			printf("-r: neigborhood radius must be an integer > 0 \n");
			exit(1);
		}
		break;
	   }
	   case 'u':
	   {
		algo->UF = atof(optarg);
		if (algo->UF < 0.0 || algo->UF > 1.0)
		{
			printf("-u: pso unification factor [0, 1] \n");
			exit(1);
		}
		break;
	   }
	   case 'b':
	   {
		*PSOparset = atoi(optarg);
		if (*PSOparset != 1 &&  *PSOparset != 2)
		{
			printf("-b: possible choice of parameters 1 or 2 \n");
			exit(1);
		}
		break;
	   }
	   case 'n':
	   {
		algo->seed = atoi(optarg);
		if (algo->seed == 0 )
		{
			printf("-n: random seed must be an integer > 0. -1 indicates seed from system clock \n");
			exit(1);
		}
		break;
	   }
	   case 'U':
	   {
		algo->R3use = atoi(optarg);
		if (algo->R3use != 0  && algo->R3use != 1)
		{
			printf("-U: use mutation 0/1\n");
			exit(1);
		}
		break;
	   }
	   case 'M':
	   {
		algo->R3mean = atof(optarg);
		break;
	   }
	   case 'S':
	   {
		algo->R3std = atof(optarg);
		break;
	   }
	   case 'F':
	   {
		algo->F = atof(optarg);
		break;
	   }
	   case 'C':
	   {
		algo->CR = atof(optarg);
		break;
	   }
	   case 'v':
	   {
		algo->oper = atoi(optarg);
		if (algo->oper != 1 && algo->oper != 2 && algo->oper != 3 && algo->oper != 4 && algo->oper != 5)
		{
			printf("-v: differential evolution strategy [1-5]\n");
			exit(1);
		}
		break;
	   }
	   case 'O':
	   {
		if (optarg != NULL)
			strcpy(foutpre, optarg);		
		else
		{
			printf("-O: empty output filename \n");
			exit(1);
		}
		break;
	   }	
           case 'p':
	   {
		*noprint = atoi(optarg);
		if (*noprint != 0 && *noprint != 1)
		{
			printf("-p: Print levels [0: no print to file, 1: complete print to file] \n");
			exit(1);
		}
		break;
    	   }
           case 'x':
	   {
		*printsolflag = 1;
		break;
    	   }
           case 'l':
	   {
		algo->memetic = atoi(optarg);
#ifdef NOMERLIN
		printf("---------------------------------------------------------------------------\n");
		printf("The software is compiled with NOMERLIN flag and does not allow local searches\n");
		printf("---------------------------------------------------------------------------\n");
		algo->memetic = 0;
#endif
		if (algo->memetic != 0 && algo->memetic != 1 && algo->memetic != 2 && algo->memetic != 3)
		{
			printf("-l: memetic choices [0, 1, 2, 3] \n");
			exit(1);			
		}
		break;
    	   }
	   case 'c':
	   {
		algo->Vscale = atof(optarg);
		if (algo->Vscale < 0.0)
		{
			printf("-c: Scaling factor must be greater than zero \n");
			exit(1);			
		}
		break;
	   }
	   case 'P':
	   {
		algo->prob = atof(optarg);
		if (algo->prob < 0.0 || algo->prob > 1.0)
		{
			printf("-P: Local search probability must be a number in [0, 1] \n");
			exit(1);			
		}
		break;
	   }
	   case 'D':
	   {
		algo->dump = atof(optarg);
		if (algo->dump < 0 )
		{
			printf("-D: The number of iterations to write temporary results must be > 0 \n");
			exit(1);			
		}
		break;
	   }
	   case 'E':
	   {
		EPS = atof(optarg);
		if (EPS < 0)
		{
			printf("-E: The tolerance must be a small positive number. \n");
			exit(1);			
		}
		break;
	   }

	   case 'X':
	   {
		XEPS = atof(optarg);
		if (XEPS < 0 )
		{
			printf("-X: X-tolerance must be a small positive number \n");
			exit(1);			
		}
		break;
	   }
	   case 'Y':
	   {
		FEPS = atof(optarg);
		if (FEPS < 0 )
		{
			printf("-Y: F-tolerance must be a small positive number  \n");
			exit(1);			
		}
		break;
	   }
	   case 'G':
	   {
		GEPS = atof(optarg);
		if (GEPS < 0 )
		{
			printf("-G: G-tolerance must be a small positive number  \n");
			exit(1);			
		}
		break;
	   }
	   case 'y':
	   {
		strcpy(TMPIN, optarg);
		if ((fp = fopen(TMPIN, "r")) == 0 )
		{
			printf("-y: File %s containing Merlin commands does not exist \n", optarg);			
			exit(1);			
		}
		fclose(fp);
		break;
	   }
	   case 'z':
	   {
	   	if (optarg != "")
			strcpy(TMPOUT, optarg);
		else	
			strcpy(TMPOUT, "out.dat");
			 
		break;
	   }
	   case 'W':
	   {
	 	algo->dump_swarm = 1;	
		if (optarg == "")
			strcpy(TMPOUFILE, ".psode");	
		else
			strcpy(TMPOUFILE, optarg);	

		break;
	   }
	   case 'R':
	   {
	 	algo->restart = 1;	
		if (optarg == "")
			strcpy(TMPINFILE, ".psode");	
		else
			strcpy(TMPINFILE, optarg);	

		break;
	   }
	   case 'V':
	   {
	 	algo->new_vel = 1;	
		break;
	   }
	   case 'h':
	   {
		printf("Usage: %s [OPTION ...] \n", argv[0]);
		printf("\nDescription: \n");
		printf("mempsode is a specialized global optimization software that implements a memetic swarm \n");
		printf("intelligence scheme combining Particle Swarm Optimization (PSO) or Differential Evolution (DE) with  \n");
		printf("effective local optimization algorithms. The software provides a state of the art   \n");
		printf("implementations of PSO and DE and uses Merlin environment for local optimization. \n");
		printf("In the following NUM represents an integer number, FLOAT a floating point number and\n");
		printf("BIN a binary number (0 or 1). \n\n");
	
		printf("  -a --algorithm=ALG \n \t\t\t Switch between PSO and DE ");
		printf(" (example: -a pso or -a de)  \n\n");
		printf("--------------------------------------------------------------------------------\n");
		printf("Common options \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("  -d NUM --dimension=NUM \n \t\t\t Sets the dimensionality of the problem [default: 2]\n\n");
		printf("  -s NUM --swarm-size=NUM \n \t\t\t Sets the size of the swarm  [default: 10*dimension]\n\n");
		printf("  -e NUM --num-experiments=NUM \n \t\t\t Number of experiments. The algorithm is restarted for every experiment [default: 1]\n\n");
		printf("  -l NUM --memetic=NUM \n \t\t\t Local search - hybrid scheme  [default: 0]\n\n");
		printf("  -n SEED --seed=SEED \n \t\t\t Random number generator seed  [default: 1]\n");
		printf("\t\t\t If SEED=-1, use system clock to randomize \n\n");
		printf("  -o NUM --objective=NUM \n \t\t\t Sets the objective function's id. [default: 0]\n\n");
		printf("  -E FLOAT --EPS=FLOAT \n \t\t\t Specifies a small positive number [default:1.e-8] \n\n");
		printf("  -X FLOAT --XEPS=FLOAT \n \t\t\t Sets the X-tolerance threshold  [default:1.e-8] \n\n");	
		printf("  -Y FLOAT --YEPS=FLOAT \n \t\t\t Sets the function value threshold [default:1.e-6] \n");	
		printf("  -G FLOAT --GEPS=FLOAT \n \t\t\t Sets the gradient norm threshold [default:1.e-5] \n");	
		printf("  -y FILE --infile=FILE \n \t\t\t Sets the file containing Merlin instructions for local search [default:\"in.dat\"] \n");	
		printf("  -z FILE --outfile=FILE \n \t\t\t Sets the file containing the output from Merlin local search [default:\"out.dat\"] \n");	
		printf("  -h  --help \n \t\t\t Displays this information\n");	
		printf("--------------------------------------------------------------------------------\n");
		printf("Termination options \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("  -f NUM --max-fun-evals=NUM\n \t\t\t Sets the maximum number of function evaluations [default: -1(disabled)]\n\n");
		printf("  -g NUM --max-grad-evals=NUM\n \t\t\t Sets the maximum number of gradient evaluations [default: -1(disabled)]\n\n");
		printf("  -i NUM --max-iters=NUM\n \t\t\t Sets the maximum number of iterations [default: -1](disabled)\n\n");
		printf("  -t FLOAT --target-value=FLOAT\n \t\t\t Sets the desired target value. The algorithm stops if it approximates \n \t\t\t the target value closer than YEPS  [default: -inf]\n\n");
		printf(" If none of the above options are set, the default termination critetion is set to maximum \n");
		printf(" number of function evaluations (<10000*dimension).  \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("Printout options \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("  -O PREFIX --out-file-prefix=PREFIX \n \t\t\t Sets the prefix for output files [default: \"Default\"]\n\n");
		printf("  -p NUM --file-output NUM \n \t\t\t Controls file printing. 0: no print to files, 1: complete print to files [default: 1]\n\n");	
		printf("  -x --print-solution \n \t\t\t Enables printing of the solution vector on the screen,  \n");
		printf("\t\t\t 0: The solution vector is not printed, \n");
		printf("\t\t\t 1: The solution vector is printed [default: 0]\n\n");
 		printf("  -D NUM --dump-iteration=NUM \n \t\t\t Specifies the number of iterations to write  temporary results  [default: 50]\n");
		printf("\t\t\t 0: No print at all \n\n");
		printf("  -R FILE --restore-from-file=FILE \n \t\t\t Reads initial swarm parameters (positions, velocities) \n \t\t\t from a previously created file [default:\".psode\"] \n\n");	
		printf("  -W FILE --dump-from-file=FILE \n \t\t\t Creates a file that stores the paremeters of the swarm. \n \t\t\t This file can be used to restart the process [default:\".psode\"] \n");	
		printf("  -V \t\t\t When a swarm is read from a file the velocity is randomly re-initialized \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("PSO options \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("  -r NUM --radius=NUM \n \t\t\t Radius for neighborhood in local PSO [default: 1]\n\n");
		printf("  -u FLOAT --unification-factor=FLOAT \n \t\t\t Unification factor [default: 0.1]\n\n");
		printf("  -U BIN --use-mutation=BIN \n \t\t\t Flag to indicate usage of mutation [default: 0]\n");
		printf("\t\t\t 0: don't use mutation, 1: use mutation \n\n");
		printf("  -M FLOAT --mutation-mean=FLOAT \n \t\t\t Mutation mean [default: 0.0]\n\n");
		printf("  -S FLOAT --mutation-std=FLOAT \n \t\t\t Mutation standard deviation [default: 0.0]\n\n");
		printf("  -P FLOAT --prob-local-search=FLOAT \n \t\t\t Local search probability  [default: 0.01] \n\n");
 		printf("  -c FLOAT --velocity-scaling=FLOAT \n \t\t\t Scaling factor for velocity initialization [default: 0.5]\n\n");
 		printf("  -b NUM --pso-params=NUM \n \t\t\t Switches between two different sets of PSO parameters (chi, c1, c2) \n \t\t\t [default: 1(chi=,]\n\n");
		printf("--------------------------------------------------------------------------------\n");
		printf("DE options \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("  -F FLOAT --F-param=FLOAT \n \t\t\t F parameter of DE [default: 0.5]\n\n");
		printf("  -C FLOAT --C-param=FLOAT \n \t\t\t CR parameter of DE [default: 0.7]\n\n");
		printf("  -v NUM --de-strategy=NUM \n \t\t\t Switch DE variant [default: 0]\n");
		printf("\t\t\t 1:  DE/BEST/1/BIN \n");
		printf("\t\t\t 2:  DE/RAND/1/BIN \n");
		printf("\t\t\t 3:  E/RAND-TO-BEST/1/BIN \n");
		printf("\t\t\t 4:  DE/BEST/2/BIN\n");
		printf("\t\t\t 5:  DE/RAND/2/BIN \n\n");
		printf("--------------------------------------------------------------------------------\n");
		printf("Examples \n");
		printf("--------------------------------------------------------------------------------\n");
		printf("./mempsode -a pso --dimension=10 --swarm-size=50 -f 10000 -l 2 \n");
		printf("\t\tApply the second memetic strategy for a particle swarm consisting of \n");
		printf("\t\t50 particles in a 10-dimensional search space uning a maximum budget of \n");
		printf("\t\t20000 function evaluations. \n\n");
		printf("./mempsode -a de -de-strategy=4 -d 10 -s 50 --max-fun-evals=20000 \n");
		printf("\t\tApply differential evolution on a swarm of 50 particles using \n");
		printf("\t\tthe 4th DE strategy (DE/BEST/2/BIN) in a 10-dimensional search \n");
		printf("\t\tspace for maximum of 10000 function evaluations. \n\n");
		exit(0);
		break;
	   }
	   default:
	   {
		printf("Error: Unknown command line option. Terminating... \n");
	    exit(1);
	   }
    }

   if (prob->maxfev == -1)
	prob->maxfev = 100000*prob->dim;


}


void write_results(int i, PROBLEM_par prob, RUN_STATS	stats, FILE *fout1, FILE *fout2, FILE *fout3, int noprint)
{
	int j;
		//---------------
		// WRITE RESULTS
		//---------------
		printf("%3i - %i -", i+1, stats.success );
		printf(" %E - %7li - %8li - %5i - %6i ", stats.best_f, stats.iter, stats.fev, stats.localsearch, stats.gev);
		printf("- [%7li,%8li,%7li,%7li] - %6li - %7.2f \n", stats.lasthit, stats.lastf, stats.lastg, stats.lastl, stats.BPup, stats.cpu);

		if (noprint == 1)
		{		
			fprintf(fout1,"%d - %d - [", i+1, stats.success);
			if (printsolflag == 1)
			{
				for (j=0; j<prob.dim; j++) 
					fprintf(fout1,"%E ", stats.best_x[j]);
			}
			fprintf(fout1, "] - %le - %d - %d - %d - %d - %d - %d - %d - %d - %d - %f\n", stats.best_f, 
				(int)stats.iter, (int)stats.fev, (int)stats.localsearch, (int) stats.gev, (int)stats.lasthit, (int)stats.lastf, (int)stats.lastg, (int)stats.lastl, (int)stats.BPup, stats.cpu);
		
			fprintf(fout2,"%d %d ", i+1, stats.success);
			fprintf(fout3,"%d ", i+1);
			for (j=0; j<prob.dim; j++) 
				fprintf(fout3,"%E ", stats.best_x[j]);
			fprintf(fout3,"\n");
			fprintf(fout2, "%le %d %d %d %d %d %d %d %d %d %f\n", stats.best_f, 
				(int)stats.iter, (int)stats.fev, (int)stats.localsearch, (int)stats.gev, (int)stats.lasthit, (int)stats.lastf, (int)stats.lastg, (int)stats.lastl, (int)stats.BPup, stats.cpu);	

		}
}









