C
C =========================================================================================================
C C. Voglis, Department of Computer Science, 
C           Univeristy of Ioannina, Greece
C           
C File: objective.f 
C Usage:  Function funmin is the objective function minized by MERLIN. 
C         We provide a definition of function funmin() that call an
C         objective function wrapper written in C.
C         We also provide dummy definitions to other necessary subroutines.
C Last Modified: 1/12/2010
C =========================================================================================================
C
C  ---------------------------------------------------------------------
	SUBROUTINE CHEMIN ( GN, EPS, POINT, NOD, VAL, XLL, XRL, GRAD,
     &                    NOC, NGR )
C  ---------------------------------------------------------------------
C
C  Description:
C    Calculates the root mean square gradient, for the
C    components that do not "coincide" with one of their margins.
C    If all variables coincide with one of their margins then 
C    the rms Gradient (GN) is set to -1.
C
C  ---------------------------------------------------------------------
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C  ---------------------------------------------------------------------
C  Arguments:
	DIMENSION POINT(NOD), GRAD(NOD), XLL(NOD), XRL(NOD)
C  ---------------------------------------------------------------------
C
	CALL DERMER(POINT,VAL,GRAD,NOC,NGR,1)
	GN = 0.D0
	DNOD = 0
	DO 1 I = 1,NOD
		YL = ABS( XLL(I) - POINT(I) )
		YR = ABS( XRL(I) - POINT(I) )
		YLX = MAX( ABS( XLL(I) ), ABS( POINT(I) ) )*EPS
		YRX = MAX( ABS( XRL(I) ), ABS( POINT(I) ) )*EPS
		IF (YL .GT. YLX .AND. YR .GT. YRX ) THEN
			GN = GN + GRAD(I)**2
			DNOD = DNOD + 1
		END IF
1	CONTINUE
	GN = SQRT(GN/DNOD)
	END
